'
' ####################
' #####  PROLOG  #####
' ####################
'
' A program demonstrating dynamic memory access with the accessor macros
' remember to turn on m4!
'
VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "xsx"				' Extended standard library
'	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

' IMPORT "gdi32"			' gdi32.dll
'	IMPORT "user32"		  ' user32.dll
'	IMPORT "kernel32"	  ' kernel32.dll
'	IMPORT "shell32"		' shell32.dll
'	IMPORT "msvcrt"		  ' msvcrt.dll

' include the macros
m4_include(`accessors.m4')

'the linked node
TYPE LINKEDNODE
	XLONG	.data
	XLONG	.next
	XLONG	.prev
END TYPE

'the linked list data structure
TYPE LINKEDLIST
	XLONG	.head
	XLONG	.tail
	XLONG	.cItems
END TYPE

DECLARE FUNCTION Entry ()
'linked list functions
DECLARE FUNCTION LinkedList_Init (LINKEDLIST @list)
DECLARE FUNCTION LinkedList_Append (LINKEDLIST @list, data)
DECLARE FUNCTION LinkedList_Delete (LINKEDLIST @list, item, @data)
DECLARE FUNCTION LinkedList_GetFirstNode (LINKEDLIST list, LINKEDNODE @node)
DECLARE FUNCTION LinkedList_GetNode (LINKEDLIST list, item, LINKEDNODE @node)

'Declare the accessors for STRING and LINKEDNODE
DeclareAccess(STRING)
DeclareAccess(LINKEDNODE)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()
	'we need a local linked list and linked node
	LINKEDLIST list
	LINKEDNODE node

	'always remember to initialise the accessors
	STRING_Init()
	LINKEDNODE_Init()

	'initialise our simple linked list
	LinkedList_Init(@list)

	'give the user some assistance
	PRINT "Enter a blank line for help"

	'this is just a simple command loop
	done = $$FALSE
  DO WHILE !done
		XstParseStringToStringArray (INLINE$ ("\nEnter a command: "), " ", @s$[])
		IF UBOUND(s$[]) < 0 THEN
			DIM s$[0]
			s$[0] = ""
		END IF

		SELECT CASE UCASE$(TRIM$(s$[0]))
			CASE "ADD"
				'add an item to the end of the linked list
				IF UBOUND(s$[]) < 1 THEN
					PRINT "Bad Parameter, correct syntax is ADD \"<string>\""
				ELSE
					'append an item to the local linked list
					LinkedList_Append (@list, STRING_New (s$[1]))
				END IF
			CASE "DELETE"
				IF UBOUND(s$[]) < 1 THEN
					PRINT "Bad Parameter, correct syntax is DELETE <item>"
				ELSE
					'delete the specified item from the linked list
					LinkedList_Delete(@list, XLONG(s$[1]), @data)

					'delete the data for the item
					STRING_Delete (data)
				END IF
			CASE "FIND"
				IF UBOUND(s$[]) < 1 THEN
					PRINT "Bad Parameter, correct syntax is FIND \"<string>\""
				ELSE
					'get the first node
					id = LinkedList_GetFirstNode (list, @node)
					'walk the list
					s$[1] = UCASE$(TRIM$(s$[1]))
					i = 0
					n = 0
					DO WHILE id != 0
						STRING_Get(node.data, @stri$)
						IF UCASE$(TRIM$(stri$)) = s$[1] THEN
							PRINT "Found at position ";i
							INC n
						END IF
						INC i
						id = LINKEDNODE_Get(node.next, @node)
					LOOP
					PRINT "Found ";n;" occurences"
				END IF
			CASE "LIST"
				'get the first node
				id = LinkedList_GetFirstNode (list, @node)
				'walk the list
				DO WHILE id != 0
					STRING_Get(node.data, @stri$)
					PRINT stri$
					id = LINKEDNODE_Get(node.next, @node)
				LOOP
			CASE "QUIT"
				done = $$TRUE

			CASE ELSE
				PRINT "Commands\n"
				PRINT "ADD    - add a string"
				PRINT "DELETE - delete a string"
				PRINT "FIND   - find a string"
				PRINT "LIST   - list a string"
				PRINT "QUIT   - quit the program"
		END SELECT
	LOOP

END FUNCTION
'
' #############################
' #####  LinkedList_Init  #####
' #############################
' Initialise a linked list data structure
' @list = the list to initialise
' returns 0
FUNCTION LinkedList_Init (LINKEDLIST list)
	LINKEDNODE node
	list.head = LINKEDNODE_New(node)
	list.tail = list.head
	list.cItems = 0
END FUNCTION
'
' ###############################
' #####  LinkedList_Append  #####
' ###############################
' Add an item to a liked list
' @list = the list to append to
' data = an XLONG containing the data, often this will be the id of another object
' returns 0
FUNCTION LinkedList_Append (LINKEDLIST list, data)
	LINKEDNODE node

	node.data = data
	node.next = 0
	node.prev = list.tail
	newNode = LINKEDNODE_New(node)

	'update the tail node
	LINKEDNODE_Get(list.tail, @node)
	node.next = newNode
	LINKEDNODE_Update(list.tail, @node)

	'replace the tail
	list.tail = newNode
	INC list.cItems
END FUNCTION
'
' ###############################
' #####  LinkedList_Delete  #####
' ###############################
' Deletes an item from a list.  Items are indexed from 0
' @list = the list to delete from
' item = the index of the item to delete
' @data = the data to delete
' returns $$TRUE on success or $$FALSE on fail
FUNCTION LinkedList_Delete (LINKEDLIST list, item, data)
	LINKEDNODE node
	LINKEDNODE node2

	'get the node
	id = LinkedList_GetNode (list, item, @node)
	IFZ id THEN RETURN $$FALSE

	'node now contains the node we want to delete
	IF id = list.tail THEN list.tail = node.prev

	LINKEDNODE_Get(node.next, @node2)
	node2.prev = node.prev
	LINKEDNODE_Update(node.next, @node2)

	LINKEDNODE_Get(node.prev, @node2)
	node2.next = node.next
	LINKEDNODE_Update(node.prev, @node2)

	data = node.data

	LINKEDNODE_Delete(id)

	RETURN $$TRUE
END FUNCTION
'
' #####################################
' #####  LinkedList_GetFirstNode  #####
' #####################################
' Gets the first node in the list
' list = the list to get from
' @node = will contain the node structure
' returns the id of the node.  If this is 0, the list is empty
FUNCTION LinkedList_GetFirstNode (LINKEDLIST list, LINKEDNODE node)
	'the first node i the list is the node after the head node
	LINKEDNODE_Get(list.head, @node)
	ret = node.next
	LINKEDNODE_Get(node.next, @node)
	RETURN ret
END FUNCTION
'
' ################################
' #####  LinkedList_GetNode  #####
' ################################
' Fetch the node at the given location
' list = the list
' item = the 0 based index of the node to get
' node = the variable to store the node
' returns the id of the node, or 0 on fail
FUNCTION LinkedList_GetNode (LINKEDLIST list, item, LINKEDNODE node)

	IF item < 0 THEN RETURN 0

	LINKEDNODE_Get(list.head, @node)

	FOR i = 0 TO item
		id = node.next
		IF id = 0 THEN RETURN 0
		LINKEDNODE_Get(id, @node)
	NEXT

	RETURN id
END FUNCTION

DefineAccess(STRING)
DefineAccess(LINKEDNODE)

END PROGRAM