'
' ####################
' #####  PROLOG  #####
' ####################
'
' A simple dbase III+/IV file access library.
' Adapted from PB code by Ian Cairns
' Use and modify at your own risk.
'
'DBF FILE STRUCTURE
'~~~~~~~~~~~~~~~~~~

'BYTES DESCRIPTION
'00    FoxBase+, FoxPro, dBaseIII+, dBaseIV, no memo - 0x03
'      FoxBase+, dBaseIII+ with memo - 0x83
'      FoxPro with memo - 0xF5
'      dBaseIV with memo - 0x8B
'      dBaseIV with SQL Table - 0x8E

'01-03 Last update, format YYYYMMDD **correction: it is YYMMDD**
'04-07 Number of records in file (32-bit number)
'08-09 Number of bytes in header (16-bit number)
'10-11 Number of bytes in record (16-bit number)
'12-13 Reserved, fill with 0x00
'14    dBaseIV flag, incomplete transaction
'      Begin Transaction sets it to 0x01
'      End Transaction or RollBack reset it to 0x00

'15    Encryption flag, encrypted 0x01 else 0x00
'      Changing the flag does not encrypt or decrypt the records

'16-27 dBaseIV multi-user environment use
'28    Production index exists - 0x01 else 0x00
'29    dBaseIV language driver ID
'30-31 Reserved fill with 0x00
'32-n  Field Descriptor array
'n+1   Header Record Terminator - 0x0D

'FIELD DESCRIPTOR ARRAY TABLE
'BYTES DESCRIPTION
'0-10  Field Name ASCII padded with 0x00
'11    Field Type Identifier (see table)
'12-15 Displacement of field in record
'16    Field length in bytes
'17    Field decimal places
'18-19 Reserved
'20    dBaseIV work area ID
'21-30 Reserved
'31    Field is part of production index - 0x01 else 0x00

'FIELD IDENTIFIER TABLE
'ASCII DESCRIPTION
' C     Character
' D     Date, format YYYYMMDD
' F     Floating Point
' G     General - FoxPro addition
' L     Logical, T:t,F:f,Y:y,N:n,?-not initialized
' M     Memo (stored as 10 digits representing the dbt block number)
' N     Numeric
' P     Picture - FoxPro addition

'  Note all dbf field records begin with a deleted flag field.
'  If record is deleted - 0x2A (asterisk) else 0x20 (space)
'  End of file is marked with 0x1A

'
' ####################
' ##### PROLOG #####
' ####################
'
PROGRAM "xdb3"
VERSION "0.0001"
CONSOLE

IMPORT "xst"
IMPORT "xsx"
IMPORT "xio"
IMPORT "kernel32"

EXPORT
PACKED DB3HEADERSTRUCTURE
	UBYTE  .version
	UBYTE  .year
	UBYTE  .month
	UBYTE  .day
	ULONG  .numRecords
	USHORT .headerLength
	USHORT .recordLength
	USHORT .reserved
	UBYTE  .fTransaction
	UBYTE  .fEncrypted
	XLONG  .reserved1
	XLONG  .reserved2
	XLONG  .reserved3
	UBYTE  .fIndex
	UBYTE  .langID
	USHORT .reserved4
END TYPE

PACKED DB3HEADERINFO
	UBYTE  .version
	UBYTE  .memo
	UBYTE  .year
	UBYTE  .month
	UBYTE  .day
	ULONG  .numRecords
	USHORT .numFields
	USHORT .headerLength
	USHORT .recordLength
	UBYTE  .fIndex
END TYPE

PACKED DB3FIELDSTRUCTURE
	STRING * 10 .fieldName
	UBYTE       .fieldEnd
	STRING * 1  .fieldType
	ULONG       .memOffset
	UBYTE       .fieldLength
	UBYTE       .decimals
	USHORT      .reserved1
	UBYTE       .id
	STRING * 10 .reserved2
	UBYTE       .fIndex
END TYPE

PACKED DB3FIELDINFO
	USHORT      .fieldOffset
	STRING * 10 .fieldName
	STRING * 1  .fieldType
	UBYTE       .fieldLength
	UBYTE       .decimals
	UBYTE       .fIndex
END TYPE

DECLARE FUNCTION XDb3 ()
DECLARE FUNCTION Db3PutHeaderInfo (fhandle, DB3HEADERINFO hi)
DECLARE FUNCTION Db3GetHeaderInfo (fhandle, DB3HEADERINFO hi)
DECLARE FUNCTION Db3OpenFile (@fhandle, file$, DB3HEADERINFO hi, DB3FIELDINFO fi[])
DECLARE FUNCTION Db3CreateFile (@fhandle, file$, DB3HEADERINFO hi, DB3FIELDINFO fi[])
DECLARE FUNCTION Db3GetTimeStamp (@year, @month, @day, @date$)
DECLARE FUNCTION Db3CloseFile (fhandle, DB3HEADERINFO hi)
DECLARE FUNCTION Db3RecordDeleted (record$)
DECLARE FUNCTION Db3GetField$ (record$, DB3FIELDINFO fi)
DECLARE FUNCTION Db3GetRecord (fhandle, ULONG recNum, @record$, DB3HEADERINFO hi)
DECLARE FUNCTION Db3PutRecord (fhandle, recNum, record$, DB3HEADERINFO hi)
DECLARE FUNCTION Db3SetField (@record$, value$, DB3FIELDINFO fi)
DECLARE FUNCTION Db3FormatField$ (value$, DB3FIELDINFO fi)
DECLARE FUNCTION Db3InspectFile (file$, fRecords)
DECLARE FUNCTION Db3CreateField (fieldName$, fieldType$, fieldLength, decimals, DB3FIELDINFO fi)
END EXPORT

$$MAX_FILE_SEEK = 4294967294

FUNCTION XDb3 ()

	DB3HEADERINFO hi
	DB3FIELDINFO fi
	DB3FIELDINFO fi[]

	IF LIBRARY(0) THEN RETURN

	DIM fi[8]

	' file name
	file$ = "xdb3test.dbf"

	' create some fields and copy to fi[] array
	Db3CreateField ("VINEYARD", "C", 50, 0, @fi)
	fi[0] = fi

	Db3CreateField ("AOC", "C", 50, 0, @fi)
	fi[1] = fi

	Db3CreateField ("CITY", "C", 30, 0, @fi)
	fi[2] = fi

	Db3CreateField ("BRAND", "C", 30, 0, @fi)
	fi[3] = fi

	Db3CreateField ("COUNTRY", "C", 30, 0, @fi)
	fi[4] = fi

	Db3CreateField ("VINTAGE", "C", 4, 0, @fi)
	fi[5] = fi

	Db3CreateField ("QUANTITY", "N", 4, 0, @fi)
	fi[6] = fi

	Db3CreateField ("PURCHASED", "D", 8, 0, @fi)
	fi[7] = fi

	Db3CreateField ("PRICE", "F", 5, 2, @fi)
	fi[8] = fi

	' create new dbf file
	err = Db3CreateFile (@fhandle, file$, @hi, @fi[])

	' add some records
	record$ = SPACE$(hi.recordLength)

	fi = fi[0]
	Db3SetField (@record$, "Domaine Du Pourra", @fi)
	fi = fi[1]
	Db3SetField (@record$, "Gigondas", @fi)
	fi = fi[2]
	Db3SetField (@record$, "Sablet", @fi)
	fi = fi[4]
	Db3SetField (@record$, "France", @fi)
	fi = fi[5]
	Db3SetField (@record$, "2002", @fi)
	fi = fi[6]
	Db3SetField (@record$, "6", @fi)
	fi = fi[7]
	Db3GetTimeStamp (@year, @month, @day, @date$)
	Db3SetField (@record$, date$, @fi)
	fi = fi[8]
	Db3SetField (@record$, "113.85", @fi)

	recNum = 1
	hi.numRecords = recNum
	Db3PutRecord (fhandle, recNum, record$, @hi)
	INC recNum

	fi = fi[0]
	Db3SetField (@record$, "Chateau Hourbanon", @fi)
	fi = fi[1]
	Db3SetField (@record$, "Medoc", @fi)
	fi = fi[2]
	Db3SetField (@record$, "Prignac", @fi)
	fi = fi[4]
	Db3SetField (@record$, "France", @fi)
	fi = fi[5]
	Db3SetField (@record$, "2002", @fi)
	fi = fi[6]
	Db3SetField (@record$, "12", @fi)
	fi = fi[7]
	Db3GetTimeStamp (@year, @month, @day, @date$)
	Db3SetField (@record$, date$, @fi)
	fi = fi[8]
	Db3SetField (@record$, "9.95", @fi)

	hi.numRecords = recNum
	Db3PutRecord (fhandle, recNum, record$, @hi)
	INC recNum

	fi = fi[0]
	Db3SetField (@record$, "Abbaye Frontfroide", @fi)
	fi = fi[1]
	Db3SetField (@record$, "Corbieres", @fi)
	fi = fi[2]
	Db3SetField (@record$, "Narbonne", @fi)
	fi = fi[3]
	Db3SetField (@record$, "Deo Gratias", @fi)
	fi = fi[4]
	Db3SetField (@record$, "France", @fi)
	fi = fi[5]
	Db3SetField (@record$, "2005", @fi)
	fi = fi[6]
	Db3SetField (@record$, "3", @fi)
	fi = fi[7]
	Db3GetTimeStamp (@year, @month, @day, @date$)
	Db3SetField (@record$, date$, @fi)
	fi = fi[8]
	Db3SetField (@record$, "12.55", @fi)

	hi.numRecords = recNum
	Db3PutRecord (fhandle, recNum, record$, @hi)
	INC recNum

	fi = fi[0]
	Db3SetField (@record$, "Domaine Galuval", @fi)
	fi = fi[1]
	Db3SetField (@record$, "Cotes du Rhone Village", @fi)
	fi = fi[2]
	Db3SetField (@record$, "Cairanne", @fi)
	fi = fi[3]
	Db3SetField (@record$, "Petit Coeur", @fi)
	fi = fi[4]
	Db3SetField (@record$, "France", @fi)
	fi = fi[5]
	Db3SetField (@record$, "2005", @fi)
	fi = fi[6]
	Db3SetField (@record$, "6", @fi)
	fi = fi[7]
	Db3GetTimeStamp (@year, @month, @day, @date$)
	Db3SetField (@record$, date$, @fi)
	fi = fi[8]
	Db3SetField (@record$, "118.99", @fi)

	hi.numRecords = recNum
	Db3PutRecord (fhandle, recNum, record$, @hi)
	INC recNum

	' finished, close file
	err = Db3CloseFile (fhandle, @hi)

	Db3InspectFile (file$, 1)

INLINE$ ("Press any key to quit")

END FUNCTION
'
' ##############################
' #####  Db3PutHeaderInfo  #####
' ##############################
'
' Converts DB3HEADERINFO into a DB3HEADERSTRUCTURE type and writes it to file.
'
FUNCTION Db3PutHeaderInfo (fhandle, DB3HEADERINFO hi)

	DB3HEADERSTRUCTURE hs

	IF fhandle < 3 THEN RETURN ($$TRUE)

  hs.version   = 3   										' Fixed identifier for dBase file
  IF hi.memo > 0 THEN
	  hs.version = SET(hs.version, 1, 7)  ' FoxBase+, dBaseIII+ with memo - 0x83
		hs.version = SET(hs.version, 1, 3)  ' dBaseIV with memo - 0x8B
  END IF
  hs.year      		= hi.year
  hs.month     		= hi.month
  hs.day       		= hi.day
  hs.numRecords   = hi.numRecords
  hs.headerLength = hi.headerLength
  hs.recordLength = hi.recordLength
  hs.fIndex   		= hi.fIndex

	SEEK (fhandle, 0)
  WRITE [fhandle], hs

END FUNCTION
'
' ##############################
' #####  Db3GetHeaderInfo  #####
' ##############################
'
' Reads DB3HEADERSTRUCTURE type info and converts it to DB3HEADERINFO type
' Returns 1 for incorrect version,
' Returns 2 for file encrypted
' Returns -1 for file error
'
FUNCTION Db3GetHeaderInfo (fhandle, DB3HEADERINFO hi)

	DB3HEADERSTRUCTURE hs

	IF fhandle < 3 THEN RETURN ($$TRUE)

	SEEK (fhandle, 0)
	READ [fhandle], hs

	' ***  fill header info  ***

	hi.version = hs.version
	IF (hi.version AND 3) <> 3 THEN RETURN (1) 	' not a dbase III+/IV file

	IF hs.fEncrypted THEN RETURN (2) 						' file encrypted

	SELECT CASE hs.version
		CASE 0x3, 0x8e        : hi.memo = 0
		CASE 0x83, 0xf5, 0x8b : hi.memo = 1
		CASE ELSE             : hi.memo = 0
	END SELECT
	hi.year 				= hs.year
	hi.month 				= hs.month
	hi.day 					= hs.day
	hi.numRecords 	= hs.numRecords
	hi.numFields 		= (hs.headerLength - 33) \ 32
	hi.headerLength = hs.headerLength
	hi.recordLength = hs.recordLength
	hi.fIndex 			= hs.fIndex

END FUNCTION
'
' #########################
' #####  Db3OpenFile  #####
' #########################
'
' Open an existing .dbf file.
' Pass the file name to the function and receive a handle to the open file
' as well as the file Header+Field Information.
'
FUNCTION Db3OpenFile (@fhandle, file$, DB3HEADERINFO hi, DB3FIELDINFO fi[])

	DB3FIELDSTRUCTURE fs

	dot = RINSTR(file$, ".")
	IFZ dot THEN RETURN (-2)
	file$ = LCASE$(file$)
	IFZ INSTR(file$, ".dbf") THEN RETURN (-2)

	fhandle = OPEN (file$, $$RD)
	IF (fhandle < 3) THEN RETURN ($$TRUE)

	err = Db3GetHeaderInfo (fhandle, @hi)
	IF err THEN
		CLOSE (fhandle)
		RETURN err
	END IF

	REDIM fi[hi.numFields-1]

	offSet = 0
	FOR x = 0 TO hi.numFields - 1
		READ [fhandle], fs
		fi[x].fieldOffset = offSet
		fi[x].fieldName 	= fs.fieldName
		fi[x].fieldType 	= fs.fieldType
		fi[x].fieldLength = fs.fieldLength
		fi[x].decimals 		= fs.decimals
		fi[x].fIndex 			= fs.fIndex
		offSet = offSet + fs.fieldLength
	NEXT x

END FUNCTION
'
' ###########################
' #####  Db3CreateFile  #####
' ###########################
'
' Create a new .dbf file.
' Pass the desired structures. If possible, the file is created
' and the file handle is returned, or the handle is returned as 0.
' File is dateStamped with the current date.

' file is created without checking for existance, correct filename characters
'       or header structure.
'
FUNCTION Db3CreateFile (@fhandle, file$, DB3HEADERINFO hi, DB3FIELDINFO fi[])

	DB3FIELDSTRUCTURE fs

	dot = RINSTR(file$, ".")
	IFZ dot THEN RETURN (-2)
	file$ = LCASE$(file$)
	IFZ INSTR(file$, ".dbf") THEN RETURN (-2)

	IFZ fi[] THEN RETURN (-3)

  hi.numFields 		= UBOUND(fi[]) + 1
  hi.headerLength = hi.numFields * 32 + 33
  hi.version   		= 3

	err = Db3GetTimeStamp (@year, @month, @day, @date$)
	hi.year  = year
	hi.month = month
	hi.day   = day

  hi.numRecords = 0
  hi.recordLength = 1
  FOR i = 0 TO hi.numFields - 1
    hi.recordLength = hi.recordLength + fi[i].fieldLength
  NEXT i

	fhandle = OPEN (file$, $$RWNEW)
	IF fhandle < 3 THEN RETURN ($$TRUE)

  err = Db3PutHeaderInfo (fhandle, @hi)
	IF err THEN
		? "Db3CreateFile error: err"
		RETURN err
	END IF

  offset = 1 '2
  FOR i = 0 TO hi.numFields - 1
    fs.fieldName 		= fi[i].fieldName
    fs.fieldType 		= fi[i].fieldType
    fs.fieldLength  = fi[i].fieldLength
    fs.decimals  		= fi[i].decimals
    fs.fIndex  			= fi[i].fIndex
		WRITE [fhandle], fs
    fi[i].fieldOffset = offset
    offset = offset + fi[i].fieldLength
  NEXT i

' EOF marker

	SEEK (fhandle, hi.headerLength)
	eof$ = CHR$(0x1A)
	WRITE [fhandle], eof$

END FUNCTION
'
' #############################
' #####  Db3GetTimeStamp  #####
' #############################
'
' Get a current system time and convert it to
' two digit values for year, month, and day.
' Also return a formatted date string (D type)
'
FUNCTION Db3GetTimeStamp (@year, @month, @day, @date$)

	GetSystemTime (&systime)
	format$ = "yyMMdd"
	d$ = NULL$(LEN(format$)+1)
	IFZ GetDateFormatA (NULL, NULL, &systime, &format$, &d$,	LEN(d$)) THEN RETURN ($$TRUE)
	year  = XLONG(LEFT$(d$, 2))
	month = XLONG(MID$(d$, 3, 2))
	day   = XLONG(RIGHT$(d$, 2))

	format$ = "yyyyMMdd"
	date$ = NULL$(LEN(format$)+1)
	IFZ GetDateFormatA (NULL, NULL, &systime, &format$, &date$,	LEN(date$)) THEN RETURN ($$TRUE)

END FUNCTION
'
' ##########################
' #####  Db3CloseFile  #####
' ##########################
'
' Update file timestamp, number of records, and then close file fhandle.
'
FUNCTION Db3CloseFile (fhandle, DB3HEADERINFO hi)

	DB3HEADERSTRUCTURE hs

	IF fhandle < 3 THEN RETURN ($$TRUE)

	SEEK (fhandle, 0)
	READ [fhandle], hs

	' update number of records
	hs.numRecords = hi.numRecords

	' update date
	Db3GetTimeStamp (@year, @month, @day, @date$)
	hs.year  = year
	hs.month = month
	hs.day   = day

	' write header back to file
	SEEK (fhandle, 0)
	WRITE [fhandle], hs

	' write end of file byte
  recordOffset = (hi.numRecords * hi.recordLength) + hi.headerLength
  eof$ = CHR$(0x1A)
	SEEK (fhandle, recordOffset)
	WRITE [fhandle], eof$

	' close file
	CLOSE (fhandle)

END FUNCTION
'
' ##############################
' #####  Db3RecordDeleted  #####
' ##############################
'
' Returns $$FALSE if record is not marked for deletion,
' $$TRUE if it is marked for deletion.
'
FUNCTION Db3RecordDeleted (record$)

	IF LEFT$(record$) = "*" THEN RETURN ($$TRUE)

END FUNCTION
'
' #########################
' #####  Db3GetField  #####
' #########################
'
' Extracts a specific field from dBase record$
'
FUNCTION Db3GetField$ (record$, DB3FIELDINFO fi)

	IFZ record$ THEN RETURN ("")
	RETURN MID$(record$, fi.fieldOffset+2, fi.fieldLength)

END FUNCTION
'
' ##########################
' #####  Db3GetRecord  #####
' ##########################
'
' Returns record number recNum in record$.
'
FUNCTION Db3GetRecord (fhandle, ULONG recNum, @record$, DB3HEADERINFO hi)

	ULONG recordOffset

	IF fhandle < 3 THEN RETURN ($$TRUE)

  recordOffset = ((recNum - 1) * hi.recordLength) + hi.headerLength
	IF recordOffset > ($$MAX_FILE_SEEK - hi.recordLength) THEN RETURN ($$TRUE)

	SEEK (fhandle, recordOffset)
	record$ = NULL$(hi.recordLength)
	READ [fhandle], record$

END FUNCTION
'
' ##########################
' #####  Db3PutRecord  #####
' ##########################
'
' Writes record$ as record number recNum to file.
'
FUNCTION Db3PutRecord (fhandle, recNum, record$, DB3HEADERINFO hi)

  ULONG recordOffset

	IFZ record$ THEN RETURN ($$TRUE)
	IF fhandle < 3 THEN RETURN ($$TRUE)

  recordOffset = ((recNum - 1) * hi.recordLength) + hi.headerLength
	IF recordOffset > ($$MAX_FILE_SEEK - hi.recordLength) THEN RETURN ($$TRUE)

	SEEK (fhandle, recordOffset)
  WRITE [fhandle], record$

END FUNCTION
'
' #########################
' #####  Db3PutField  #####
' #########################
'
' Stuffs value$ into position in record$ as defined by DB3FIELDINFO fi
'
FUNCTION Db3SetField (@record$, value$, DB3FIELDINFO fi)

	IFZ record$ THEN RETURN

	field$ = Db3FormatField$ (value$, fi)
	record$ = STUFF$ (record$, field$, fi.fieldOffset+1, fi.fieldLength)

END FUNCTION
'
' #############################
' #####  Db3FormatField$  #####
' #############################
'
' Formats a field value$ based on field type found in DB3FIELDINFO fi
'
FUNCTION Db3FormatField$ (value$, DB3FIELDINFO fi)

	otext$ = SPACE$(fi.fieldLength)

  SELECT CASE fi.fieldType
		' Character fields are left justified
    CASE "C"
      otext$ = STUFF$ (otext$, value$, 1)

		' Numeric/Float fields are right justified around the decimal
    CASE "N", "F"
			' use fi.decimals
			IF fi.decimals THEN
				pre = fi.fieldLength - fi.decimals
				format$ = "+" + CHR$('#', pre) + "." + CHR$('#', fi.decimals)
				f! = SINGLE(value$)
				ctext$ = FORMAT$ (format$, f!)
				' replace positive sign with space
				XstReplace (@ctext$, "+", " ", 1)
				XstStripChars (@ctext$, ".", 0)
				ctext$ = TRIM$(ctext$)

			' No decimals allowed
      ELSE
				format$= "+" + CHR$('#', fi.fieldLength - 1)
				ctext$ = FORMAT$ (format$, SINGLE(value$))
				' replace positive sign with space
				XstReplace (@ctext$, "+", " ", 1)
				ctext$ = TRIM$(ctext$)

      END IF
			pos = LEN(otext$) - LEN(ctext$) + 1
			otext$ = STUFF$ (otext$, ctext$, pos)

    CASE "D", "L", "M"  'D = date, L = Logical, M = memo
			ctext$ = TRIM$ (value$)
			otext$ = STUFF$ (otext$, ctext$, LEN(otext$) - LEN(ctext$) + 1)

  END SELECT

	RETURN otext$

END FUNCTION
'
' ############################
' #####  Db3InspectFile  #####
' ############################
'
' Display file header info and field info
' IF fRecords is $$TRUE, then display all data records
'
FUNCTION Db3InspectFile (file$, fRecords)

DB3HEADERINFO hi
DB3FIELDINFO fi[]
DB3FIELDINFO fi

	err = Db3OpenFile (@fhandle, file$, @hi, @fi[])
	IF err THEN
		? "Unable to open "; file$; ": error "; err
		RETURN ($$TRUE)
	END IF

	PRINT
	PRINT "================================================="
	PRINT "Begin DBF Inspection for " + file$
	PRINT "================================================="
	PRINT

	PRINT "File/Table Information:"
	PRINT
	PRINT "       ", "         ", "Number ", "Header", "Record"
	PRINT "Version", "Edit Date", "Records", "Length", "Length"
	PRINT "-------", "---------", "-------", "------", "------"

	PRINT FORMAT$ ("<<<<<<<", HEXX$(hi.version));
	PRINT TAB(2); FORMAT$ ("<<<<<<<<<", STRING$(hi.month) + "/" + STRING$(hi.day) + "/" + STRING$(hi.year));
	PRINT TAB(2); FORMAT$ ("<<<<<<<", STRING$(hi.numRecords));
	PRINT TAB(2); FORMAT$ ("<<<<<<", STRING$(hi.headerLength));
	PRINT TAB(3); FORMAT$ ("<<<<<<", STRING$(hi.recordLength))

	PRINT
	PRINT
	PRINT "Table Structure (Fields):"
	PRINT
	PRINT "Name      ", "Type", "Offset", "Length", "Decimals"
	PRINT "----------", "----", "------", "------", "--------"

	upp = UBOUND (fi[])
	FOR x = 0 TO upp
		PRINT FORMAT$("<<<<<<<<<<", fi[x].fieldName);
		PRINT TAB(3); fi[x].fieldType;
		PRINT TAB(6); FORMAT$("<<<<<<", STRING$ (fi[x].fieldOffset));
		PRINT TAB(3); FORMAT$("<<<<<<", STRING$ (fi[x].fieldLength));
		PRINT TAB(3); STRING$ (fi[x].decimals)
	NEXT x

	IF fRecords THEN

		bufsize = ((upp+5) * (hi.numRecords)) + 500
		XioSetConsoleBufferSize (XioGetStdOut(), 80, bufsize)

		PRINT
		PRINT
		PRINT "Records (Data):"
		PRINT

		fnameFormat$ = ">>>>>>>>>>: "

		FOR x = 1 TO hi.numRecords

			Db3GetRecord (fhandle, x, @record$, @hi)

			' extract data for each field
			PRINT
			PRINT ">>>>>>>>>>: RECORD "; STRING$(x)

			FOR y = 0 TO hi.numFields - 1
				fi = fi[y]
				f$ = Db3GetField$ (record$, @fi)

				SELECT CASE fi[y].fieldType
					CASE "C", "L", "D"

					CASE "N", "F"
						IF fi[y].decimals THEN
							tempFormat$ = CHR$('#', fi[y].fieldLength - fi[y].decimals)
							format$ = "+" + tempFormat$ + "." + CHR$ (35, fi[y].decimals)
							' insert .
							dot = fi[y].fieldLength - fi[y].decimals
							f$ = LEFT$(f$, dot) + "." + MID$(f$, dot+1)
							f$ = FORMAT$ (format$, SINGLE (f$))
							XstStripChars (@f$, "+ ", 0)
						ELSE
							format$ = "+" + CHR$ (35, fi[y].fieldLength)
							f$ = FORMAT$ (format$, SINGLE (f$))
							XstStripChars (@f$, "+ ", 0)
						END IF

				END SELECT

				PRINT FORMAT$ (fnameFormat$, fi[y].fieldName); f$

			NEXT y
		NEXT x
	END IF

END FUNCTION
'
' ############################
' #####  Db3CreateField  #####
' ############################
'
' Create a field by filling a DB3FIELDINFO fi struct
'
FUNCTION Db3CreateField (fieldName$, fieldType$, fieldLength, decimals, DB3FIELDINFO fi)

	IFZ fieldName$ THEN RETURN ($$TRUE)
	IF LEN (fieldName$) > 10 THEN RETURN ($$TRUE)
	IFZ fieldLength THEN RETURN ($$TRUE)
	IF LEN (fieldType$) > 1 THEN RETURN ($$TRUE)

	SELECT CASE fieldType$
		CASE "C"				: decimals = 0
		CASE "D"				: fieldLength = 8 	: decimals = 0
		CASE "F", "N"		:
		CASE "L"				: fieldLength = 1 	: decimals = 0
		CASE "M"				: fieldLength = 10	: decimals = 0
		CASE ELSE : RETURN $$TRUE
	END SELECT

	fi.fieldName 		= fieldName$
	fi.fieldType 		= fieldType$
	fi.fieldLength 	= fieldLength
	fi.decimals			= decimals

END FUNCTION
END PROGRAM